/*
 * Copyright (c) 2015, Victor Nazarov <asviraspossible@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation and/or
 *     other materials provided with the distribution.
 *
 *  3. Neither the name of the copyright holder nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.sviperll.adt4j.model;

import com.github.sviperll.meta.MemberAccess;
import com.helger.jcodemodel.JMethod;
import java.util.Set;
import java.util.TreeSet;

/**
 *
 * @author Victor Nazarov <asviraspossible@gmail.com>
 */
class PredicateConfigutation {
    private final Set<String> cases = new TreeSet<String> ();
    private final MemberAccess accessLevel;
    PredicateConfigutation(JMethod interfaceMethod, MemberAccess accessLevel) {
        this.accessLevel = accessLevel;
        cases.add(interfaceMethod.name());
    }

    void put(JMethod interfaceMethod, MemberAccess accessLevel) throws PredicateConfigurationException {
        if (this.accessLevel != accessLevel)
            throw new PredicateConfigurationException("Conflicting access level for predicate");
        cases.add(interfaceMethod.name());
    }

    MemberAccess accessLevel() {
        return accessLevel;
    }

    boolean isTrueFor(JMethod interfaceMethod) {
        return cases.contains(interfaceMethod.name());
    }
}
